# [Flow][]

Write more understandable Haskell.

[![Version][]](https://hackage.haskell.org/package/flow)
[![Build][]](https://travis-ci.org/tfausak/flow)
[![Dependencies][]](http://packdeps.haskellers.com/feed?needle=flow)

---

Flow is a package that provides functions and operators for writing more
understandable Haskell. It's an alternative to some common idioms like
[`($)`][] for function application and [`(.)`][] for function composition.

-   [Requirements](#requirements)
-   [Installation](#installation)
-   [Usage](#usage)

## Requirements

Flow requires a Haskell compiler. It is tested with recent versions of GHC, but
older or different compilers should be acceptable. For installation with Cabal,
Flow requires at least Cabal 1.8.

## Installation

To add Flow as a dependency to your package, add it to your Cabal file.

```
build-depends: flow ==1.0.*
```

For other use cases, install it with Cabal.

``` sh
$ cabal install 'flow ==1.0.*'
```

Flow uses [Semantic Versioning][]. See [the change log][] for a detailed list
of changes.

## Usage

Flow is designed to be imported unqualified. It does not export anything that
conflicts with [the base package][].

``` hs
import Flow
```

Here is a quick overview of the functions and operators that Flow provides.

Flow            | Base
--------------- | -------------
`apply x f`     | `f x`
`x |> f`        | `x & f`
`f <| x`        | `f $ x`
`compose f g x` | `g (f x)`
`f .> g`        | `f >>> g`
`g <. f`        | `g . f`
`apply' x f`    | `seq x (f x)`
`x !> f`        | -
`f <! x`        | `f $! x`

For more information about Flow, please read [the Haddock documentation][].

[flow]: http://taylor.fausak.me/flow/
[version]: https://img.shields.io/hackage/v/flow.svg?label=version&style=flat-square
[build]: https://img.shields.io/travis/tfausak/flow/master.svg?label=build&style=flat-square
[dependencies]: https://img.shields.io/hackage-deps/v/flow.svg?label=dependencies&style=flat-square
[`($)`]: http://hackage.haskell.org/package/base-4.8.0.0/docs/Prelude.html#v:-36-
[`(.)`]: http://hackage.haskell.org/package/base-4.8.0.0/docs/Prelude.html#v:.
[semantic versioning]: http://semver.org/spec/v2.0.0.html
[the change log]: CHANGELOG.md
[the base package]: http://hackage.haskell.org/package/base
[the haddock documentation]: https://hackage.haskell.org/package/flow/docs/Flow.html
